@@ -390,10 +390,7 @@ static int panel_mipi_dbi_spi_probe(struct spi_device *spi)
 
 	spi_set_drvdata(spi, drm);
 
-	if (bpp == 16)
-		drm_client_setup_with_fourcc(drm, DRM_FORMAT_RGB565);
-	else
-		drm_client_setup_with_fourcc(drm, DRM_FORMAT_RGB888);
+	drm_client_setup(drm, NULL);
 
 	return 0;
 }
